@@ -43,13 +43,13 @@ from optparse import OptionParser
 import rospkg
 
 from . import create_default_installer_context, get_default_installer
-from .core import RosdepInternalError, InstallFailed
+from . import __version__
+from .core import RosdepInternalError, InstallFailed, UnsupportedOs, InvalidData
 from .installers import RosdepInstaller
-from .lookup import RosdepLookup, ResolutionError, RosdepConflict
+from .lookup import RosdepLookup, ResolutionError
+from .sources_list import update_sources_list, get_sources_cache_dir,\
+     download_default_sources_list, get_sources_list_dir
 
-class UnsupportedOs(Exception):
-    pass
-    
 class UsageError(Exception):
     pass
 
@@ -57,26 +57,33 @@ _usage = """usage: rosdep [options] <command> <args>
 
 Commands:
 
-rosdep check <packages>...
-  will check if the dependencies of package(s) have been met.
+rosdep check <stacks-and-packages>...
+  check if the dependencies of package(s) have been met.
 
-rosdep install <packages>...
-  will generate a bash script and then execute it.
+rosdep install <stacks-and-packages>...
+  generate a bash script and then execute it.
 
-rosdep db <packages>...
-  will generate the dependency database for package(s) and print
-  it to the console (note that the database will change depending
-  on which package(s) you query.
+rosdep db
+  generate the dependency database and print it to the console.
 
-rosdep keys <packages>...
-  will list the rosdep keys that the packages depend on.
+rosdep init
+  initialize rosdep sources in /etc/ros/rosdep.  May require sudo.
+  
+rosdep keys <stacks-and-packages>...
+  list the rosdep keys that the packages depend on.
 
+rosdep resolve <rosdeps>
+  resolve <rosdeps> to system dependencies
+  
+rosdep update
+  update the local rosdep database based on the rosdep sources.
+  
 rosdep what-needs <rosdeps>...
-  will print a list of packages that declare a rosdep on (at least
+  print a list of packages that declare a rosdep on (at least
   one of) <rosdeps>
 
 rosdep where-defined <rosdeps>...
-  will print a list of yaml files that declare a rosdep on (at least
+  print a list of yaml files that declare a rosdep on (at least
   one of) <rosdeps>
 """
 
@@ -111,12 +118,6 @@ Please go to the rosdep page [1] and file a bug report with the message below.
 %s
 """%(e.message), file=sys.stderr)
         sys.exit(1)
-    except RosdepConflict as e:
-        print("""
-ERROR: conflict in rosdep data files
-%s
-"""%(e), file=sys.stderr)
-        sys.exit(1)        
     except ResolutionError as e:
         print("""
 ERROR: %s
@@ -143,6 +144,8 @@ def _rosdep_main(args):
                       metavar="OS_NAME:OS_VERSION", help="Override OS name and version (colon-separated), e.g. ubuntu:lucid")
     parser.add_option("--verbose", "-v", dest="verbose", default=False, 
                       action="store_true", help="verbose display")
+    parser.add_option("--version", dest="print_version", default=False, 
+                      action="store_true", help="print version and exit")
     parser.add_option("--reinstall", dest="reinstall", default=False, 
                       action="store_true", help="(re)install all dependencies, even if already installed")
     parser.add_option("--include_duplicates", "-i", dest="include_duplicates", default=False, 
@@ -157,30 +160,42 @@ def _rosdep_main(args):
                       action="store_true", help="select all packages")
 
     options, args = parser.parse_args(args)
+    if options.print_version:
+        print(__version__)
+        sys.exit(0)
 
     if len(args) == 0:
         parser.error("Please enter a command")
     command = args[0]
     if not command in _commands:
         parser.error("Unsupported command %s."%command)
-    if len(args) < 2 and not options.rosdep_all:
-        parser.error("Please enter arguments for '%s'"%command)
     args = args[1:]
 
     if command in _command_rosdep_args:
         return _rosdep_args_handler(command, parser, options, args)
+    elif command in _command_no_args:
+        return _no_args_handler(command, parser, options, args)        
     else:
         return _package_args_handler(command, parser, options, args)
 
+def _no_args_handler(command, parser, options, args):
+    if args:
+        parser.error("command [%s] takes no arguments"%(command))
+    else:
+        return command_handlers[command](options)
+    
 def _rosdep_args_handler(command, parser, options, args):
+
     # rosdep keys as args
     if options.rosdep_all:
         parser.error("-a, --all is not a valid option for this command")
+    elif len(args) < 1:
+        parser.error("Please enter arguments for '%s'"%command)
     else:
         return command_handlers[command](args, options)
     
 def _package_args_handler(command, parser, options, args, rospack=None, rosstack=None):
-    # package names as args
+    # package or stack names as args.  have to convert stack names to packages.
     # - overrides to enable testing
     if rospack is None:
         rospack = rospkg.RosPack()
@@ -197,7 +212,8 @@ def _package_args_handler(command, parser, options, args, rospack=None, rosstack
             # let the loader filter the -a. This will take out some
             # packages that are catkinized (for now).
             args = loader.get_loadable_resources()
-    if not args:
+            not_found = []
+    elif not args:
         parser.error("no packages or stacks specified")
 
     val = rospkg.expand_to_packages(args, rospack, rosstack)
@@ -206,7 +222,7 @@ def _package_args_handler(command, parser, options, args, rospack=None, rosstack
     if not_found:
         raise rospkg.ResourceNotFound(not_found[0], rospack.get_ros_paths())
 
-    if not packages:
+    if 0 and not packages: # disable, let individual handlers specify behavior
         # possible with empty stacks
         print("No packages in arguments, aborting")
         return
@@ -228,6 +244,43 @@ def configure_installer_context_os(installer_context, options):
     os_version = val[val.find(':')+1:]
     installer_context.set_os_override(os_name, os_version)
     
+def command_init(options):
+    data = download_default_sources_list()
+    # reuse path variable for error message
+    path = get_sources_list_dir()
+    try:
+        if not os.path.exists(path):
+            os.makedirs(path)
+        path = os.path.join(path, '20-default.list')
+        if os.path.exists(path):
+            print("ERROR: default sources list file already exists:\n\t%s\nPlease delete if you wish to re-initialize"%(path))
+            sys.exit(1)
+        with open(path, 'w') as f:
+            f.write(data)
+        print("Wrote %s"%(path))
+        print("Recommended: please run 'rosdep update' now")
+    except IOError as e:
+        print("ERROR: cannot create %s:\n\t%s"%(path, e), file=sys.stderr)
+        sys.exit(2)        
+    except OSError as e:
+        print("ERROR: cannot create %s:\n\t%s\nPerhaps you need to run 'sudo rosdep init' instead"%(path, e), file=sys.stderr)
+        sys.exit(3)
+    
+def command_update(options):
+    def update_success_handler(data_source):
+        print("Hit %s"%(data_source.url))
+    def update_error_handler(data_source, exc):
+        print("ERROR: unable to process source [%s]:\n\t%s"%(data_source.url, exc), file=sys.stderr)
+    try:
+        print("reading in sources list data from %s"%(get_sources_list_dir()))
+        update_sources_list(success_handler=update_success_handler,
+                            error_handler=update_error_handler)
+        print("updated cache in %s"%(get_sources_cache_dir()))
+    except InvalidData as e:
+        print("ERROR: invalid sources list file:\n\t%s"%(e))
+    except IOError as e:
+        print("ERROR: error loading sources list:\n\t%s"%(e))
+    
 def command_keys(lookup, packages, options):
     lookup = _get_default_RosdepLookup(verbose=options.verbose)
     rosdep_keys = []
@@ -268,7 +321,7 @@ def command_check(lookup, packages, options):
 
 def error_to_human_readable(error):
     if isinstance(error, rospkg.ResourceNotFound):
-        return "Missing resource [%s]"%(str(error.args[0]))
+        return "Missing resource %s"%(str(error))
     elif isinstance(error, ResolutionError):
         return str(error.args[0])
     else:
@@ -326,7 +379,7 @@ def _compute_depdb_output(lookup, packages, options):
             output = output + "<<<< %s -> %s >>>>\n"%(rosdep, resolved)
     return output
     
-def command_db(lookup, resources, options):
+def command_db(lookup, options):
     # exact same setup logic as command_resolve, should possibly combine
     lookup = _get_default_RosdepLookup(verbose=options.verbose)
     installer_context = create_default_installer_context(verbose=options.verbose)
@@ -342,22 +395,18 @@ def command_db(lookup, resources, options):
     print("OS NAME: %s"%os_name)
     print("OS VERSION: %s"%os_version)
     errors = []
-    for resource_name in resources:
-        if len(resources) > 1:
-            print("RESOURCE: %s"%resource_name)
-        print("DB [key -> resolution]")
-        view = lookup.get_rosdep_view_for_resource(resource_name)
-        if view is None:
-            continue
-        for rosdep_name in view.keys():
-            try:
-                d = view.lookup(rosdep_name)
-                inst_key, rule = d.get_rule_for_platform(os_name, os_version, installer_keys, default_key)
-                resolved = installer.resolve(rule)
-                resolved_str = " ".join(resolved)
-                print ("%s -> %s"%(rosdep_name, resolved_str))
-            except ResolutionError as e:
-                errors.append(e)
+    print("DB [key -> resolution]")
+    # db does not leverage the resource-based API
+    view = lookup.get_rosdep_view(DEFAULT_VIEW_KEY, verbose=options.verbose)
+    for rosdep_name in view.keys():
+        try:
+            d = view.lookup(rosdep_name)
+            inst_key, rule = d.get_rule_for_platform(os_name, os_version, installer_keys, default_key)
+            resolved = installer.resolve(rule)
+            resolved_str = " ".join(resolved)
+            print ("%s -> %s"%(rosdep_name, resolved_str))
+        except ResolutionError as e:
+            errors.append(e)
 
     for error in errors:
         print("WARNING: %s"%(error_to_human_readable(error)), file=sys.stderr)
@@ -399,7 +448,8 @@ def command_resolve(args, options):
     configure_installer_context_os(installer_context, options)
 
     installer, installer_keys, default_key, \
-            os_name, os_version = get_default_installer(lookup, installer_context=installer_context, verbose=options.verbose)
+            os_name, os_version = get_default_installer(installer_context=installer_context,
+                                                        verbose=options.verbose)
     for rosdep_name in args:
         if len(args) > 1:
             print("#ROSDEP[%s]"%rosdep_name)
@@ -408,8 +458,9 @@ def command_resolve(args, options):
         if not view_names:
             raise ResolutionError(rosdep_name, None, os_name, os_version, "Could not find definition for rosdep [%s]"%rosdep_name)
         for view_name, origin in view_names:
-            print("#[%s:%s]"%(view_name, origin))
-            view = lookup.get_rosdep_view(view_name)
+            if len(view_names) > 1:
+                print("#[%s]"%(origin))
+            view = lookup.get_rosdep_view(view_name, verbose=options.verbose)
             if view is None:
                 continue
             d = view.lookup(rosdep_name)
@@ -429,6 +480,8 @@ command_handlers = {
     'what-needs': command_what_needs,
     'where-defined': command_where_defined,
     'resolve': command_resolve,
+    'init': command_init,
+    'update': command_update,
 
     # backwards compat
     'what_needs': command_what_needs,
@@ -438,6 +491,8 @@ command_handlers = {
 
 # commands that accept rosdep names as args
 _command_rosdep_args = ['what-needs', 'what_needs', 'where-defined', 'where_defined', 'resolve']
+# commands that take no args
+_command_no_args = ['update', 'init', 'db']
 
 _commands = command_handlers.keys()
 
